; Direct Digital Synthesizer
; Ikagawa/Ikagawa Lab.
; 1999.10.15 Ver.1.00

        .device AT90S4414

        .equ    INT0addr        = $001
        .equ    INT1addr        = $002
        .equ    ICP1addr        = $003
        .equ    OC1Aaddr        = $004
        .equ    OC1Baddr        = $005
        .equ    OVF1addr        = $006
        .equ    OVF0addr        = $007
        .equ    SPIaddr         = $008
        .equ    URXCaddr        = $009
        .equ    UDREaddr        = $00a
        .equ    UTXCaddr        = $00b
        .equ    ACIaddr         = $00c

        .equ    PORTA           = $1b
        .equ    DDRA            = $1a
        .equ    PINA            = $19
        .equ    PORTB           = $18
        .equ    DDRB            = $17
        .equ    PINB            = $16
        .equ    PORTC           = $15
        .equ    DDRC            = $14
        .equ    PINC            = $13
        .equ    PORTD           = $12
        .equ    DDRD            = $11
        .equ    PIND            = $10




        .cseg

        .org $0000
        rjmp    RESET                   ; Reset handler

        .org INT0addr
        reti                            ; External interrupt0 handler

        .org INT1addr
        reti                            ; External interrupt1 handler

        .org ICP1addr
        reti                            ; Input Capture1 interrupt handler

        .org OC1Aaddr
        reti                            ; Output Capture1 interrupt handler

        .org OC1Baddr
        reti                            ; Output Capture1 interrupt handler

        .org OVF1addr
        reti                            ; Overflow1 interrupt handler

        .org OVF0addr
        reti                            ; Overflow0 interrupt handler

        .org SPIaddr
        reti                            ; Overflow0 interrupt handler

        .org URXCaddr
        reti                            ; UART receive complete interrupt handler

        .org UDREaddr
        reti                            ; UART data register empty interrupt handler

        .org UTXCaddr
        reti                            ; UART transmit complete interrupt handler

        .org ACIaddr
        reti                            ; Analog comparator interrupt handler




RESET:

PortInitialize:
        ldi     r16,$00
        out     DDRA,r16                ; porta = input
        ldi     r16,$ff
        out     PORTA,r16               ; porta = internal pull up activate

        ldi     r16,$00
        out     DDRB,r16                ; portb = input
        ldi     r16,$ff
        out     PORTB,r16               ; portb = internal pull up activate

        ldi     r16,$ff
        out     DDRC,r16                ; portc = output
        ldi     r16,$00
        out     PORTC,r16               ; portc = $00

        ldi     r16,$00
        out     DDRD,r16                ; portd = input
        ldi     r16,$ff
        out     PORTD,r16               ; portd = internal pull up activate




DdsInitialize:
        in      r1,PINB                 ; dip sw (low byte) read
        com     r1
        in      r2,PIND                 ; dip sw (high byte) read
        com     r2
        clr     r3                      ; 

        lsl     r1                      ; x 2
        rol     r2
        rol     r3
        lsl     r1                      ; x 4
        rol     r2
        rol     r3
        lsl     r1                      ; x 8
        rol     r2
        rol     r3
        lsl     r1                      ; x 16
        rol     r2
        rol     r3
        lsl     r1                      ; x 32
        rol     r2
        rol     r3

        ldi     r28,$00                 ; phase accumulator low byte
        ldi     r29,$00                 ; phase accumulator middle byte
        ldi     r30,$00                 ; phase accumulator high byte / sine table pointer low byte
        ldi     r31,$0e                 ; sine table pointer high byte

DdsLoop:
        add     r28,r1                  ; 1  1/16
        adc     r29,r2                  ; 1  2/16
        adc     r30,r3                  ; 1  3/16
        lpm                             ; 3  6/16 (z) -> r0
        out     PORTC,r0                ; 1  7/16

        in      r4,PINB                 ; 1  8/16
        com     r4                      ; 1  9/16
        in      r5,PIND                 ; 1 10/16
        com     r5                      ; 1 11/16
        clr     r6                      ; 1 12/16

        lsl     r4                      ; 1 13/16 x  2
        rol     r5                      ; 1 14/16
        rol     r6                      ; 1 15/16

        nop                             ; 1 16/16


        add     r28,r1                  ; 1  1/16
        adc     r29,r2                  ; 1  2/16
        adc     r30,r3                  ; 1  3/16
        lpm                             ; 3  6/16 (z) -> r0
        out     PORTC,r0                ; 1  7/16

        lsl     r4                      ; 1  8/16 x  4
        rol     r5                      ; 1  9/16
        rol     r6                      ; 1 10/16

        lsl     r4                      ; 1 11/16 x  8
        rol     r5                      ; 1 12/16
        rol     r6                      ; 1 13/16

        lsl     r4                      ; 1 14/16 x 16
        rol     r5                      ; 1 15/16
        rol     r6                      ; 1 16/16


        add     r28,r1                  ; 1  1/16
        adc     r29,r2                  ; 1  2/16
        adc     r30,r3                  ; 1  3/16
        lpm                             ; 3  6/16 (z) -> r0
        out     PORTC,r0                ; 1  7/16

        lsl     r4                      ; 1  8/16 x 32
        rol     r5                      ; 1  9/16
        rol     r6                      ; 1 10/16

        mov     r1,r4                   ; 1 11/16
        mov     r2,r5                   ; 1 12/16
        mov     r3,r6                   ; 1 13/16

        nop                             ; 1 14/16

        rjmp    DdsLoop                 ; 2 16/16




        .org    $0700

SineTable:

        .db     $80,$83,$86,$89,$8c,$8f,$92,$95
        .db     $98,$9b,$9e,$a2,$a5,$a7,$aa,$ad
        .db     $b0,$b3,$b6,$b9,$bc,$be,$c1,$c4
        .db     $c6,$c9,$cb,$ce,$d0,$d3,$d5,$d7
        .db     $da,$dc,$de,$e0,$e2,$e4,$e6,$e8
        .db     $ea,$eb,$ed,$ee,$f0,$f1,$f3,$f4
        .db     $f5,$f6,$f8,$f9,$fa,$fa,$fb,$fc
        .db     $fd,$fd,$fe,$fe,$fe,$ff,$ff,$ff
        .db     $ff,$ff,$ff,$ff,$fe,$fe,$fe,$fd
        .db     $fd,$fc,$fb,$fa,$fa,$f9,$f8,$f6
        .db     $f5,$f4,$f3,$f1,$f0,$ee,$ed,$eb
        .db     $ea,$e8,$e6,$e4,$e2,$e0,$de,$dc
        .db     $da,$d7,$d5,$d3,$d0,$ce,$cb,$c9
        .db     $c6,$c4,$c1,$be,$bc,$b9,$b6,$b3
        .db     $b0,$ad,$aa,$a7,$a5,$a2,$9e,$9b
        .db     $98,$95,$92,$8f,$8c,$89,$86,$83
        .db     $7f,$7c,$79,$76,$73,$70,$6d,$6a
        .db     $67,$64,$61,$5d,$5a,$58,$55,$52
        .db     $4f,$4c,$49,$46,$43,$41,$3e,$3b
        .db     $39,$36,$34,$31,$2f,$2c,$2a,$28
        .db     $25,$23,$21,$1f,$1d,$1b,$19,$17
        .db     $15,$14,$12,$11,$0f,$0e,$0c,$0b
        .db     $0a,$09,$07,$06,$05,$05,$04,$03
        .db     $02,$02,$01,$01,$01,$00,$00,$00
        .db     $00,$00,$00,$00,$01,$01,$01,$02
        .db     $02,$03,$04,$05,$05,$06,$07,$09
        .db     $0a,$0b,$0c,$0e,$0f,$11,$12,$14
        .db     $15,$17,$19,$1b,$1d,$1f,$21,$23
        .db     $25,$28,$2a,$2c,$2f,$31,$34,$36
        .db     $39,$3b,$3e,$41,$43,$46,$49,$4c
        .db     $4f,$52,$55,$58,$5a,$5d,$61,$64
        .db     $67,$6a,$6d,$70,$73,$76,$79,$7c

;------------------------------------------------------------------------------
; [eof]
